@@ -68,7 +68,7 @@ OPTS = [
 EXCHANGES_OPTS = [
     cfg.MultiStrOpt('notification_control_exchanges',
                     default=['nova', 'glance', 'neutron', 'cinder', 'heat',
-                             'keystone', 'sahara', 'trove', 'zaqar', 'swift',
+                             'keystone', 'trove', 'zaqar', 'swift',
                              'ceilometer', 'magnum', 'dns', 'ironic', 'aodh'],
                     deprecated_group='DEFAULT',
                     deprecated_name="http_control_exchanges",
@@ -132,7 +132,11 @@ class NotificationService(cotyledon.Service):
 
         endpoints = []
         for pipe_mgr in self.managers:
-            endpoints.extend(pipe_mgr.get_main_endpoints())
+            LOG.debug("Loading manager endpoints for [%s].", pipe_mgr)
+            endpoint = pipe_mgr.get_main_endpoints()
+            LOG.debug("Loaded endpoints [%s] for manager [%s].",
+                      endpoint, pipe_mgr)
+            endpoints.extend(endpoint)
         targets = self.get_targets()
 
         urls = self.conf.notification.messaging_urls or [None]
