@@ -18,22 +18,26 @@
     similar to the idea used for handling notifications.
 """
 import copy
+import json
 import re
+import subprocess
+import time
+import xmltodict
 
 from oslo_log import log
-from oslo_utils import timeutils
 
 from requests import RequestException
 
 from ceilometer import declarative
 from ceilometer.polling import plugin_base
 from ceilometer import sample as ceilometer_sample
+from ceilometer import utils as ceilometer_utils
 
 from functools import reduce
 import operator
 import requests
 
-from six.moves.urllib import parse as url_parse
+from urllib import parse as urlparse
 
 LOG = log.getLogger(__name__)
 
@@ -45,6 +49,89 @@ def validate_sample_type(sample_type):
             % (sample_type, ceilometer_sample.TYPES))
 
 
+class XMLResponseHandler(object):
+    """This response handler converts an XML in string format to a dict"""
+
+    @staticmethod
+    def handle(response):
+        return xmltodict.parse(response)
+
+
+class JsonResponseHandler(object):
+    """This response handler converts a JSON in string format to a dict"""
+
+    @staticmethod
+    def handle(response):
+        return json.loads(response)
+
+
+class PlainTextResponseHandler(object):
+    """Response handler converts string to a list of dict [{'out'=<string>}]"""
+
+    @staticmethod
+    def handle(response):
+        return [{'out': str(response)}]
+
+
+VALID_HANDLERS = {
+    'json': JsonResponseHandler,
+    'xml': XMLResponseHandler,
+    'text': PlainTextResponseHandler
+}
+
+
+def validate_response_handler(val):
+    if not isinstance(val, list):
+        raise declarative.DynamicPollsterDefinitionException(
+            "Invalid response_handlers configuration. It must be a list. "
+            "Provided value type: %s" % type(val).__name__)
+
+    for value in val:
+        if value not in VALID_HANDLERS:
+            raise declarative.DynamicPollsterDefinitionException(
+                "Invalid response_handler value [%s]. Accepted values "
+                "are [%s]" % (value, ', '.join(list(VALID_HANDLERS))))
+
+
+def validate_extra_metadata_skip_samples(val):
+    if not isinstance(val, list) or next(
+            filter(lambda v: not isinstance(v, dict), val), None):
+        raise declarative.DynamicPollsterDefinitionException(
+            "Invalid extra_metadata_fields_skip configuration."
+            " It must be a list of maps. Provided value: %s,"
+            " value type: %s." % (val, type(val).__name__))
+
+
+class ResponseHandlerChain(object):
+    """Tries to convert a string to a dict using the response handlers"""
+
+    def __init__(self, response_handlers, **meta):
+        if not isinstance(response_handlers, list):
+            response_handlers = list(response_handlers)
+
+        self.response_handlers = response_handlers
+        self.meta = meta
+
+    def handle(self, response):
+        failed_handlers = []
+        for handler in self.response_handlers:
+            try:
+                return handler.handle(response)
+            except Exception as e:
+                handler_name = handler.__name__
+                failed_handlers.append(handler_name)
+                LOG.debug(
+                    "Error handling response [%s] with handler [%s]: %s. "
+                    "We will try the next one, if multiple handlers were "
+                    "configured.",
+                    response, handler_name, e)
+
+        handlers_str = ', '.join(failed_handlers)
+        raise declarative.InvalidResponseTypeException(
+            "No remaining handlers to handle the response [%s], "
+            "used handlers [%s]. [%s]." % (response, handlers_str, self.meta))
+
+
 class PollsterDefinitionBuilder(object):
 
     def __init__(self, definitions):
@@ -108,22 +195,42 @@ class PollsterSampleExtractor(object):
             LOG.debug("Removed key [%s] with value [%s] from "
                       "metadata set that is sent to Gnocchi.", k, k_value)
 
-    def generate_sample(self, pollster_sample, pollster_definitons=None):
+    def generate_sample(
+            self, pollster_sample, pollster_definitions=None, **kwargs):
+
         pollster_definitions =\
-            pollster_definitons or self.definitions.configurations
+            pollster_definitions or self.definitions.configurations
 
         metadata = dict()
         if 'metadata_fields' in pollster_definitions:
             for k in pollster_definitions['metadata_fields']:
-                val = self.retrieve_attribute_nested_value(pollster_sample, k)
+                val = self.retrieve_attribute_nested_value(
+                    pollster_sample, value_attribute=k,
+                    definitions=self.definitions.configurations)
 
                 LOG.debug("Assigning value [%s] to metadata key [%s].", val, k)
                 metadata[k] = val
 
         self.generate_new_metadata_fields(
             metadata=metadata, pollster_definitions=pollster_definitions)
+
+        pollster_sample['metadata'] = metadata
+        extra_metadata = self.definitions.retrieve_extra_metadata(
+            kwargs['manager'], pollster_sample, kwargs['conf'])
+
+        LOG.debug("Extra metadata [%s] collected for sample [%s].",
+                  extra_metadata, pollster_sample)
+        for key in extra_metadata.keys():
+            if key in metadata.keys():
+                LOG.warning("The extra metadata key [%s] already exist in "
+                            "pollster current metadata set [%s]. Therefore, "
+                            "we will ignore it with its value [%s].",
+                            key, metadata, extra_metadata[key])
+                continue
+            metadata[key] = extra_metadata[key]
+
         return ceilometer_sample.Sample(
-            timestamp=timeutils.isotime(),
+            timestamp=ceilometer_utils.isotime(),
             name=pollster_definitions['name'],
             type=pollster_definitions['sample_type'],
             unit=pollster_definitions['unit'],
@@ -134,15 +241,18 @@ class PollsterSampleExtractor(object):
             resource_metadata=metadata)
 
     def retrieve_attribute_nested_value(self, json_object,
-                                        value_attribute=None):
+                                        value_attribute=None,
+                                        definitions=None, **kwargs):
+        if not definitions:
+            definitions = self.definitions.configurations
 
-        attribute_key = value_attribute or self.definitions.\
-            extract_attribute_key()
+        attribute_key = value_attribute
+        if not attribute_key:
+            attribute_key = self.definitions.extract_attribute_key()
 
         LOG.debug(
             "Retrieving the nested keys [%s] from [%s] or pollster [""%s].",
-            attribute_key, json_object,
-            self.definitions.configurations["name"])
+            attribute_key, json_object, definitions["name"])
 
         keys_and_operations = attribute_key.split("|")
         attribute_key = keys_and_operations[0].strip()
@@ -153,9 +263,9 @@ class PollsterSampleExtractor(object):
             nested_keys = attribute_key.split(".")
             value = reduce(operator.getitem, nested_keys, json_object)
 
-        return self.operate_value(keys_and_operations, value)
+        return self.operate_value(keys_and_operations, value, definitions)
 
-    def operate_value(self, keys_and_operations, value):
+    def operate_value(self, keys_and_operations, value, definitions):
         # We do not have operations to be executed against the value extracted
         if len(keys_and_operations) < 2:
             return value
@@ -164,24 +274,23 @@ class PollsterSampleExtractor(object):
             if 'value' not in operation:
                 raise declarative.DynamicPollsterDefinitionException(
                     "The attribute field operation [%s] must use the ["
-                    "value] variable." % operation,
-                    self.definitions.configurations)
+                    "value] variable." % operation, definitions)
 
             LOG.debug("Executing operation [%s] against value[%s] for "
                       "pollster [%s].", operation, value,
-                      self.definitions.configurations["name"])
+                      definitions["name"])
 
             value = eval(operation.strip())
-            LOG.debug(
-                "Result [%s] of operation [%s] for pollster [%s].",
-                value, operation, self.definitions.configurations["name"])
+            LOG.debug("Result [%s] of operation [%s] for pollster [%s].",
+                      value, operation, definitions["name"])
         return value
 
 
 class SimplePollsterSampleExtractor(PollsterSampleExtractor):
 
-    def generate_single_sample(self, pollster_sample):
-        value = self.retrieve_attribute_nested_value(pollster_sample)
+    def generate_single_sample(self, pollster_sample, **kwargs):
+        value = self.retrieve_attribute_nested_value(
+            pollster_sample)
         value = self.definitions.value_mapper.map_or_skip_value(
             value, pollster_sample)
 
@@ -190,10 +299,10 @@ class SimplePollsterSampleExtractor(PollsterSampleExtractor):
 
         pollster_sample['value'] = value
 
-        return self.generate_sample(pollster_sample)
+        return self.generate_sample(pollster_sample, **kwargs)
 
-    def extract_sample(self, pollster_sample):
-        sample = self.generate_single_sample(pollster_sample)
+    def extract_sample(self, pollster_sample, **kwargs):
+        sample = self.generate_single_sample(pollster_sample, **kwargs)
         if isinstance(sample, SkippedSample):
             return sample
         yield sample
@@ -201,9 +310,11 @@ class SimplePollsterSampleExtractor(PollsterSampleExtractor):
 
 class MultiMetricPollsterSampleExtractor(PollsterSampleExtractor):
 
-    def extract_sample(self, pollster_sample):
+    def extract_sample(self, pollster_sample, **kwargs):
         pollster_definitions = self.definitions.configurations
-        value = self.retrieve_attribute_nested_value(pollster_sample)
+
+        value = self.retrieve_attribute_nested_value(
+            pollster_sample, definitions=pollster_definitions)
         LOG.debug("We are dealing with a multi metric pollster. The "
                   "value we are processing is the following: [%s].",
                   value)
@@ -223,12 +334,12 @@ class MultiMetricPollsterSampleExtractor(PollsterSampleExtractor):
                                             pollster_name, value_attribute,
                                             sub_metric_placeholder,
                                             pollster_definitions,
-                                            pollster_sample)
+                                            pollster_sample, **kwargs)
 
     def extract_sub_samples(self, value, sub_metric_attribute_name,
                             pollster_name, value_attribute,
                             sub_metric_placeholder, pollster_definitions,
-                            pollster_sample):
+                            pollster_sample, **kwargs):
 
         for sub_sample in value:
             sub_metric_name = sub_sample[sub_metric_attribute_name]
@@ -237,7 +348,7 @@ class MultiMetricPollsterSampleExtractor(PollsterSampleExtractor):
             pollster_definitions['name'] = new_metric_name
 
             actual_value = self.retrieve_attribute_nested_value(
-                sub_sample, value_attribute)
+                sub_sample, value_attribute, definitions=pollster_definitions)
 
             pollster_sample['value'] = actual_value
 
@@ -245,7 +356,8 @@ class MultiMetricPollsterSampleExtractor(PollsterSampleExtractor):
                                                 sub_metric_name):
                 continue
 
-            yield self.generate_sample(pollster_sample, pollster_definitions)
+            yield self.generate_sample(
+                pollster_sample, pollster_definitions, **kwargs)
 
     def extract_field_name_from_value_attribute_configuration(self):
         value_attribute = self.definitions.configurations['value_attribute']
@@ -392,6 +504,8 @@ class PollsterDefinitions(object):
 
     POLLSTER_VALID_NAMES_REGEXP = r"^([\w-]+)(\.[\w-]+)*(\.{[\w-]+})?$"
 
+    EXTERNAL_ENDPOINT_TYPE = "external"
+
     standard_definitions = [
         PollsterDefinition(name='name', required=True,
                            validation_regex=POLLSTER_VALID_NAMES_REGEXP),
@@ -411,7 +525,17 @@ class PollsterDefinitions(object):
         PollsterDefinition(name='user_id_attribute', default="user_id"),
         PollsterDefinition(name='resource_id_attribute', default="id"),
         PollsterDefinition(name='project_id_attribute', default="project_id"),
-        PollsterDefinition(name='headers')]
+        PollsterDefinition(name='headers'),
+        PollsterDefinition(name='timeout', default=30),
+        PollsterDefinition(name='extra_metadata_fields_cache_seconds',
+                           default=3600),
+        PollsterDefinition(name='extra_metadata_fields'),
+        PollsterDefinition(name='extra_metadata_fields_skip', default=[{}],
+                           validator=validate_extra_metadata_skip_samples),
+        PollsterDefinition(name='response_handlers', default=['json'],
+                           validator=validate_response_handler),
+        PollsterDefinition(name='base_metadata', default={})
+    ]
 
     extra_definitions = []
 
@@ -423,6 +547,7 @@ class PollsterDefinitions(object):
         self.validate_missing()
         self.sample_gatherer = PollsterSampleGatherer(self)
         self.sample_extractor = SimplePollsterSampleExtractor(self)
+        self.response_cache = {}
 
     def validate_configurations(self, configurations):
         for k, v in self.definitions.items():
@@ -463,6 +588,117 @@ class PollsterDefinitions(object):
                 "Required fields %s not specified."
                 % missing, self.configurations)
 
+    def should_skip_extra_metadata(self, skip, sample):
+        match_msg = "Sample [%s] %smatches with configured" \
+                    " extra_metadata_fields_skip [%s]."
+        if skip == sample:
+            LOG.debug(match_msg, sample, "", skip)
+            return True
+        if not isinstance(skip, dict) or not isinstance(sample, dict):
+            LOG.debug(match_msg, sample, "not ", skip)
+            return False
+
+        for key in skip:
+            if key not in sample:
+                LOG.debug(match_msg, sample, "not ", skip)
+                return False
+            if not self.should_skip_extra_metadata(skip[key], sample[key]):
+                LOG.debug(match_msg, sample, "not ", skip)
+                return False
+
+        LOG.debug(match_msg, sample, "", skip)
+        return True
+
+    def skip_sample(self, request_sample, skips):
+        for skip in skips:
+            if not skip:
+                continue
+            if self.should_skip_extra_metadata(skip, request_sample):
+                LOG.debug("Skipping extra_metadata_field gathering for "
+                          "sample [%s] as defined in the "
+                          "extra_metadata_fields_skip [%s]", request_sample,
+                          skip)
+                return True
+        return False
+
+    def retrieve_extra_metadata(self, manager, request_sample, pollster_conf):
+        extra_metadata_fields = self.configurations['extra_metadata_fields']
+        if extra_metadata_fields:
+            extra_metadata_samples = {}
+            extra_metadata_by_name = {}
+            if not isinstance(extra_metadata_fields, (list, tuple)):
+                extra_metadata_fields = [extra_metadata_fields]
+            for ext_metadata in extra_metadata_fields:
+                ext_metadata.setdefault(
+                    'extra_metadata_fields_skip',
+                    self.configurations['extra_metadata_fields_skip'])
+                ext_metadata.setdefault(
+                    'sample_type', self.configurations['sample_type'])
+                ext_metadata.setdefault('unit', self.configurations['unit'])
+                ext_metadata.setdefault(
+                    'value_attribute', ext_metadata.get(
+                        'value', self.configurations['value_attribute']))
+                ext_metadata['base_metadata'] = {
+                    'extra_metadata_captured': extra_metadata_samples,
+                    'extra_metadata_by_name': extra_metadata_by_name,
+                    'sample': request_sample
+                }
+                parent_cache_ttl = self.configurations[
+                    'extra_metadata_fields_cache_seconds']
+                cache_ttl = ext_metadata.get(
+                    'extra_metadata_fields_cache_seconds', parent_cache_ttl
+                )
+                response_cache = self.response_cache
+                extra_metadata_pollster = DynamicPollster(
+                    ext_metadata, conf=pollster_conf, cache_ttl=cache_ttl,
+                    extra_metadata_responses_cache=response_cache,
+                )
+
+                skips = ext_metadata['extra_metadata_fields_skip']
+                if self.skip_sample(request_sample, skips):
+                    continue
+
+                resources = [None]
+                if ext_metadata.get('endpoint_type'):
+                    resources = manager.discover([
+                        extra_metadata_pollster.default_discovery], {})
+                samples = extra_metadata_pollster.get_samples(
+                    manager, None, resources)
+                for sample in samples:
+                    self.fill_extra_metadata_samples(
+                        extra_metadata_by_name,
+                        extra_metadata_samples,
+                        sample)
+            return extra_metadata_samples
+
+        LOG.debug("No extra metadata to be captured for pollsters [%s] and "
+                  "request sample [%s].", self.definitions, request_sample)
+        return {}
+
+    def fill_extra_metadata_samples(self, extra_metadata_by_name,
+                                    extra_metadata_samples, sample):
+        extra_metadata_samples[sample.name] = sample.volume
+        LOG.debug("Merging the sample metadata [%s] of the "
+                  "extra_metadata_field [%s], with the "
+                  "extra_metadata_samples [%s].",
+                  sample.resource_metadata,
+                  sample.name,
+                  extra_metadata_samples)
+        for key, value in sample.resource_metadata.items():
+            if value is None and key in extra_metadata_samples:
+                LOG.debug("Metadata [%s] for extra_metadata_field [%s] "
+                          "is None, skipping metadata override by None "
+                          "value", key, sample.name)
+                continue
+            extra_metadata_samples[key] = value
+        extra_metadata_by_name[sample.name] = {
+            'value': sample.volume,
+            'metadata': sample.resource_metadata
+        }
+
+        LOG.debug("extra_metadata_samples after merging: [%s].",
+                  extra_metadata_samples)
+
 
 class MultiMetricPollsterDefinitions(PollsterDefinitions):
 
@@ -515,42 +751,96 @@ class PollsterSampleGatherer(object):
 
     def __init__(self, definitions):
         self.definitions = definitions
+        self.response_handler_chain = ResponseHandlerChain(
+            map(VALID_HANDLERS.get,
+                self.definitions.configurations['response_handlers']),
+            url_path=definitions.configurations['url_path']
+        )
+
+    def get_cache_key(self, definitions, **kwargs):
+        return self.get_request_linked_samples_url(kwargs, definitions)
+
+    def get_cached_response(self, definitions, **kwargs):
+        if self.definitions.cache_ttl == 0:
+            return
+        cache_key = self.get_cache_key(definitions, **kwargs)
+        response_cache = self.definitions.response_cache
+        cached_response, max_ttl_for_cache = response_cache.get(
+            cache_key, (None, None))
+
+        current_time = time.time()
+        if cached_response and max_ttl_for_cache >= current_time:
+            LOG.debug("Returning response [%s] for request [%s] as the TTL "
+                      "[max=%s, current_time=%s] has not expired yet.",
+                      cached_response, definitions,
+                      max_ttl_for_cache, current_time)
+            return cached_response
+
+        if cached_response and max_ttl_for_cache < current_time:
+            LOG.debug("Cleaning cached response [%s] for request [%s] "
+                      "as the TTL [max=%s, current_time=%s] has expired.",
+                      cached_response, definitions,
+                      max_ttl_for_cache, current_time)
+            response_cache.pop(cache_key, None)
+
+    def store_cached_response(self, definitions, resp, **kwargs):
+        if self.definitions.cache_ttl == 0:
+            return
+        cache_key = self.get_cache_key(definitions, **kwargs)
+        extra_metadata_fields_cache_seconds = self.definitions.cache_ttl
+        max_ttl_for_cache = time.time() + extra_metadata_fields_cache_seconds
+
+        cache_tuple = (resp, max_ttl_for_cache)
+        self.definitions.response_cache[cache_key] = cache_tuple
 
     @property
     def default_discovery(self):
         return 'endpoint:' + self.definitions.configurations['endpoint_type']
 
     def execute_request_get_samples(self, **kwargs):
-        resp, url = self.definitions.sample_gatherer. \
-            internal_execute_request_get_samples(kwargs)
+        return self.execute_request_for_definitions(
+            self.definitions.configurations, **kwargs)
 
-        response_json = resp.json()
-        entry_size = len(response_json)
-        LOG.debug("Entries [%s] in the JSON for request [%s] "
+    def execute_request_for_definitions(self, definitions, **kwargs):
+        if response_dict := self.get_cached_response(definitions, **kwargs):
+            url = 'cached'
+        else:
+            resp, url = self._internal_execute_request_get_samples(
+                definitions=definitions, **kwargs)
+            response_dict = self.response_handler_chain.handle(resp.text)
+            self.store_cached_response(definitions, response_dict, **kwargs)
+
+        entry_size = len(response_dict)
+        LOG.debug("Entries [%s] in the DICT for request [%s] "
                   "for dynamic pollster [%s].",
-                  response_json, url, self.definitions.configurations['name'])
+                  response_dict, url, definitions['name'])
 
         if entry_size > 0:
-            samples = self.retrieve_entries_from_response(response_json)
-            url_to_next_sample = self.get_url_to_next_sample(response_json)
+            samples = self.retrieve_entries_from_response(
+                response_dict, definitions)
+            url_to_next_sample = self.get_url_to_next_sample(
+                response_dict, definitions)
+
+            self.prepare_samples(definitions, samples, **kwargs)
+
             if url_to_next_sample:
                 kwargs['next_sample_url'] = url_to_next_sample
-                samples += self.execute_request_get_samples(**kwargs)
-
-            self.execute_id_overrides(samples)
+                samples += self.execute_request_for_definitions(
+                    definitions=definitions, **kwargs)
             return samples
         return []
 
-    def execute_id_overrides(self, samples):
-        if samples:
-            user_id_attribute = self.definitions.configurations[
-                'user_id_attribute']
-            project_id_attribute = self.definitions.configurations[
-                'project_id_attribute']
-            resource_id_attribute = self.definitions.configurations[
-                'resource_id_attribute']
-
+    def prepare_samples(
+            self, definitions, samples, execute_id_overrides=True, **kwargs):
+        if samples and execute_id_overrides:
             for request_sample in samples:
+                user_id_attribute = definitions.get(
+                    'user_id_attribute', 'user_id')
+                project_id_attribute = definitions.get(
+                    'project_id_attribute', 'project_id')
+                resource_id_attribute = definitions.get(
+                    'resource_id_attribute', 'id')
+
                 self.generate_new_attributes_in_sample(
                     request_sample, user_id_attribute, 'user_id')
                 self.generate_new_attributes_in_sample(
@@ -577,9 +867,9 @@ class PollsterSampleGatherer(object):
 
             sample[new_attribute_key] = attribute_value
 
-    def get_url_to_next_sample(self, resp):
-        linked_sample_extractor = self.definitions.configurations[
-            'next_sample_url_attribute']
+    def get_url_to_next_sample(self, resp, definitions):
+        linked_sample_extractor = definitions.get('next_sample_url_attribute')
+
         if not linked_sample_extractor:
             return None
 
@@ -591,60 +881,76 @@ class PollsterSampleGatherer(object):
                       "the configuration [%s]", resp, linked_sample_extractor)
         return None
 
-    def internal_execute_request_get_samples(self, kwargs):
-        keystone_client = kwargs['keystone_client']
-        url = self.get_request_linked_samples_url(kwargs)
+    def _internal_execute_request_get_samples(self, definitions=None,
+                                              keystone_client=None, **kwargs):
+        if not definitions:
+            definitions = self.definitions.configurations
 
-        request_arguments = self.create_request_arguments()
+        url = self.get_request_linked_samples_url(kwargs, definitions)
+        request_arguments = self.create_request_arguments(definitions)
         LOG.debug("Executing request against [url=%s] with parameters ["
                   "%s] for pollsters [%s]", url, request_arguments,
-                  self.definitions.configurations["name"])
-
+                  definitions["name"])
         resp = keystone_client.session.get(url, **request_arguments)
-
         if resp.status_code != requests.codes.ok:
             resp.raise_for_status()
         return resp, url
 
-    def create_request_arguments(self):
+    def create_request_arguments(self, definitions):
         request_args = {
             "authenticated": True
         }
-        request_headers = self.definitions.configurations['headers']
+        request_headers = definitions.get('headers', [])
         if request_headers:
             request_args['headers'] = request_headers
+        request_args['timeout'] = definitions.get('timeout', 300)
         return request_args
 
-    def get_request_linked_samples_url(self, kwargs):
+    def get_request_linked_samples_url(self, kwargs, definitions):
         next_sample_url = kwargs.get('next_sample_url')
         if next_sample_url:
             return self.get_next_page_url(kwargs, next_sample_url)
+
+        LOG.debug("Generating url with [%s] and path [%s].",
+                  kwargs, definitions['url_path'])
         return self.get_request_url(
-            kwargs, self.definitions.configurations['url_path'])
+            kwargs, definitions['url_path'])
 
     def get_next_page_url(self, kwargs, next_sample_url):
-        parse_result = url_parse.urlparse(next_sample_url)
+        parse_result = urlparse.urlparse(next_sample_url)
         if parse_result.netloc:
             return next_sample_url
         return self.get_request_url(kwargs, next_sample_url)
 
     def get_request_url(self, kwargs, url_path):
         endpoint = kwargs['resource']
-        return url_parse.urljoin(endpoint, url_path)
+        params = copy.deepcopy(
+            self.definitions.configurations.get(
+                'base_metadata', {}))
+        try:
+            url_path = eval(url_path, params)
+        except Exception:
+            LOG.debug("Cannot eval path [%s] with params [%s],"
+                      " using [%s] instead.",
+                      url_path, params, url_path)
+        return urlparse.urljoin((endpoint
+                                 if endpoint.endswith("/")
+                                 else (endpoint + "/")),
+                                url_path)
 
-    def retrieve_entries_from_response(self, response_json):
+    def retrieve_entries_from_response(self, response_json, definitions):
         if isinstance(response_json, list):
             return response_json
 
-        first_entry_name = \
-            self.definitions.configurations['response_entries_key']
+        first_entry_name = definitions.get('response_entries_key')
+
         if not first_entry_name:
             try:
                 first_entry_name = next(iter(response_json))
             except RuntimeError as e:
                 LOG.debug("Generator threw a StopIteration "
                           "and we need to catch it [%s].", e)
-        return self.definitions.sample_extractor. \
+        return self.definitions.sample_extractor.\
             retrieve_attribute_nested_value(response_json, first_entry_name)
 
 
@@ -668,6 +974,57 @@ class NonOpenStackApisPollsterDefinition(PollsterDefinitions):
         return configurations.get('module')
 
 
+class HostCommandPollsterDefinition(PollsterDefinitions):
+
+    extra_definitions = [
+        PollsterDefinition(name='endpoint_type', required=False),
+        PollsterDefinition(name='url_path', required=False),
+        PollsterDefinition(name='host_command', required=True)]
+
+    def __init__(self, configurations):
+        super(HostCommandPollsterDefinition, self).__init__(
+            configurations)
+        self.sample_gatherer = HostCommandSamplesGatherer(self)
+
+    @staticmethod
+    def is_field_applicable_to_definition(configurations):
+        return configurations.get('host_command')
+
+
+class HostCommandSamplesGatherer(PollsterSampleGatherer):
+
+    class Response(object):
+        def __init__(self, text):
+            self.text = text
+
+    def get_cache_key(self, definitions, **kwargs):
+        return self.get_command(definitions)
+
+    def _internal_execute_request_get_samples(self, definitions, **kwargs):
+        command = self.get_command(definitions, **kwargs)
+        LOG.debug('Running Host command: [%s]', command)
+        result = subprocess.getoutput(command)
+        LOG.debug('Host command [%s] result: [%s]', command, result)
+        return self.Response(result), command
+
+    def get_command(self, definitions, next_sample_url=None, **kwargs):
+        command = next_sample_url or definitions['host_command']
+        params = copy.deepcopy(
+            self.definitions.configurations.get(
+                'base_metadata', {}))
+        try:
+            command = eval(command, params)
+        except Exception:
+            LOG.debug("Cannot eval command [%s] with params [%s],"
+                      " using [%s] instead.",
+                      command, params, command)
+        return command
+
+    @property
+    def default_discovery(self):
+        return 'local_node'
+
+
 class NonOpenStackApisSamplesGatherer(PollsterSampleGatherer):
 
     @property
@@ -675,19 +1032,20 @@ class NonOpenStackApisSamplesGatherer(PollsterSampleGatherer):
         return 'barbican:' + \
                self.definitions.configurations['barbican_secret_id']
 
-    def internal_execute_request_get_samples(self, kwargs):
+    def _internal_execute_request_get_samples(self, definitions, **kwargs):
         credentials = kwargs['resource']
 
-        override_credentials = self.definitions.configurations[
-            'authentication_parameters']
+        override_credentials = definitions['authentication_parameters']
         if override_credentials:
             credentials = override_credentials
 
-        url = self.get_request_linked_samples_url(kwargs)
+        if not isinstance(credentials, str):
+            credentials = self.normalize_credentials_to_string(credentials)
 
-        authenticator_module_name = self.definitions.configurations['module']
-        authenticator_class_name = \
-            self.definitions.configurations['authentication_object']
+        url = self.get_request_linked_samples_url(kwargs, definitions)
+
+        authenticator_module_name = definitions['module']
+        authenticator_class_name = definitions['authentication_object']
 
         imported_module = __import__(authenticator_module_name)
         authenticator_class = getattr(imported_module,
@@ -696,12 +1054,12 @@ class NonOpenStackApisSamplesGatherer(PollsterSampleGatherer):
         authenticator_arguments = list(map(str.strip, credentials.split(",")))
         authenticator_instance = authenticator_class(*authenticator_arguments)
 
-        request_arguments = self.create_request_arguments()
+        request_arguments = self.create_request_arguments(definitions)
         request_arguments["auth"] = authenticator_instance
 
         LOG.debug("Executing request against [url=%s] with parameters ["
                   "%s] for pollsters [%s]", url, request_arguments,
-                  self.definitions.configurations["name"])
+                  definitions["name"])
         resp = requests.get(url, **request_arguments)
 
         if resp.status_code != requests.codes.ok:
@@ -712,9 +1070,21 @@ class NonOpenStackApisSamplesGatherer(PollsterSampleGatherer):
 
         return resp, url
 
-    def create_request_arguments(self):
+    @staticmethod
+    def normalize_credentials_to_string(credentials):
+        if isinstance(credentials, bytes):
+            credentials = credentials.decode('utf-8')
+        else:
+            credentials = str(credentials)
+        LOG.debug("Credentials [%s] were not defined as a string. "
+                  "Therefore, we converted it to a string like object.",
+                  credentials)
+        return credentials
+
+    def create_request_arguments(self, definitions):
         request_arguments = super(
-            NonOpenStackApisSamplesGatherer, self).create_request_arguments()
+            NonOpenStackApisSamplesGatherer, self).create_request_arguments(
+            definitions)
 
         request_arguments.pop("authenticated")
 
@@ -724,29 +1094,10 @@ class NonOpenStackApisSamplesGatherer(PollsterSampleGatherer):
         endpoint = self.definitions.configurations['url_path']
         if endpoint == url_path:
             return url_path
-        return url_parse.urljoin(endpoint, url_path)
-
-    def execute_request_get_samples(self, **kwargs):
-        samples = super(NonOpenStackApisSamplesGatherer,
-                        self).execute_request_get_samples(**kwargs)
-
-        if samples:
-            user_id_attribute = self.definitions.configurations[
-                'user_id_attribute']
-            project_id_attribute = self.definitions.configurations[
-                'project_id_attribute']
-            resource_id_attribute = self.definitions.configurations[
-                'resource_id_attribute']
-
-            for request_sample in samples:
-                self.generate_new_attributes_in_sample(
-                    request_sample, user_id_attribute, 'user_id')
-                self.generate_new_attributes_in_sample(
-                    request_sample, project_id_attribute, 'project_id')
-                self.generate_new_attributes_in_sample(
-                    request_sample, resource_id_attribute, 'id')
-
-        return samples
+        return urlparse.urljoin((endpoint
+                                 if endpoint.endswith("/")
+                                 else (endpoint + "/")),
+                                url_path)
 
     def generate_new_attributes_in_sample(
             self, sample, attribute_key, new_attribute_key):
@@ -768,8 +1119,10 @@ class DynamicPollster(plugin_base.PollsterBase):
     # Mandatory name field
     name = ""
 
-    def __init__(self, pollster_definitions={}, conf=None,
-                 supported_definitions=[NonOpenStackApisPollsterDefinition,
+    def __init__(self, pollster_definitions={}, conf=None, cache_ttl=0,
+                 extra_metadata_responses_cache=None,
+                 supported_definitions=[HostCommandPollsterDefinition,
+                                        NonOpenStackApisPollsterDefinition,
                                         MultiMetricPollsterDefinitions,
                                         SingleMetricPollsterDefinitions]):
         super(DynamicPollster, self).__init__(conf)
@@ -779,6 +1132,10 @@ class DynamicPollster(plugin_base.PollsterBase):
 
         self.definitions = PollsterDefinitionBuilder(
             self.supported_definitions).build_definitions(pollster_definitions)
+        self.definitions.cache_ttl = cache_ttl
+        self.definitions.response_cache = extra_metadata_responses_cache
+        if extra_metadata_responses_cache is None:
+            self.definitions.response_cache = {}
         self.pollster_definitions = self.definitions.configurations
         if 'metadata_fields' in self.pollster_definitions:
             LOG.debug("Metadata fields configured to [%s].",
@@ -794,8 +1151,9 @@ class DynamicPollster(plugin_base.PollsterBase):
     def load_samples(self, resource, manager):
         try:
             return self.definitions.sample_gatherer.\
-                execute_request_get_samples(keystone_client=manager._keystone,
-                                            resource=resource)
+                execute_request_get_samples(manager=manager,
+                                            resource=resource,
+                                            keystone_client=manager._keystone)
         except RequestException as e:
             LOG.warning("Error [%s] while loading samples for [%s] "
                         "for dynamic pollster [%s].",
@@ -811,12 +1169,16 @@ class DynamicPollster(plugin_base.PollsterBase):
         for r in resources:
             LOG.debug("Executing get sample for resource [%s].", r)
             samples = self.load_samples(r, manager)
+            if not isinstance(samples, (list, tuple)):
+                samples = [samples]
             for pollster_sample in samples:
-                sample = self.extract_sample(pollster_sample)
+                sample = self.extract_sample(
+                    pollster_sample, manager=manager,
+                    resource=r, conf=self.conf)
                 if isinstance(sample, SkippedSample):
                     continue
                 yield from sample
 
-    def extract_sample(self, pollster_sample):
+    def extract_sample(self, pollster_sample, **kwargs):
         return self.definitions.sample_extractor.extract_sample(
-            pollster_sample)
+            pollster_sample, **kwargs)
