@@ -16,28 +16,30 @@
 
 import abc
 
+from oslo_log import log
 from oslo_utils import netutils
-import six
 from stevedore import driver
 
 
-def get_publisher(url, namespace='ceilometer.publisher'):
+LOG = log.getLogger(__name__)
+
+
+def get_publisher(conf, url, namespace):
     """Get publisher driver and load it.
 
-    :param URL: URL for the publisher
+    :param url: URL for the publisher
     :param namespace: Namespace to use to look for drivers.
     """
     parse_result = netutils.urlsplit(url)
     loaded_driver = driver.DriverManager(namespace, parse_result.scheme)
-    return loaded_driver.driver(parse_result)
+    return loaded_driver.driver(conf, parse_result)
 
 
-@six.add_metaclass(abc.ABCMeta)
-class PublisherBase(object):
+class ConfigPublisherBase(object, metaclass=abc.ABCMeta):
     """Base class for plugins that publish data."""
 
-    def __init__(self, parsed_url):
-        pass
+    def __init__(self, conf, parsed_url):
+        self.conf = conf
 
     @abc.abstractmethod
     def publish_samples(self, samples):
