@@ -94,13 +94,19 @@ class Sample(object):
 
     def __init__(self, name, type, unit, volume, user_id, project_id,
                  resource_id, timestamp=None, resource_metadata=None,
-                 source=None, id=None, monotonic_time=None):
+                 source=None, id=None, monotonic_time=None,
+                 user_name=None, project_name=None):
+        if type not in TYPES:
+            raise ValueError('Unsupported type: %s')
+
         self.name = name
         self.type = type
         self.unit = unit
         self.volume = volume
         self.user_id = user_id
+        self.user_name = user_name
         self.project_id = project_id
+        self.project_name = project_name
         self.resource_id = resource_id
         self.timestamp = timestamp
         self.resource_metadata = resource_metadata or {}
@@ -118,7 +124,8 @@ class Sample(object):
     @classmethod
     def from_notification(cls, name, type, volume, unit,
                           user_id, project_id, resource_id,
-                          message, timestamp=None, metadata=None, source=None):
+                          message, timestamp=None, metadata=None, source=None,
+                          user_name=None, project_name=None):
         if not metadata:
             metadata = (copy.copy(message['payload'])
                         if isinstance(message['payload'], dict) else {})
@@ -135,7 +142,9 @@ class Sample(object):
                    resource_id=resource_id,
                    timestamp=ts,
                    resource_metadata=metadata,
-                   source=source)
+                   source=source,
+                   user_name=user_name,
+                   project_name=project_name)
 
     def set_timestamp(self, timestamp):
         self.timestamp = timestamp
