@@ -13,9 +13,9 @@
 # WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 # License for the specific language governing permissions and limitations
 # under the License.
+from unittest import mock
 
 import fixtures
-import mock
 
 from ceilometer.compute.virt import inspector as virt_inspector
 from ceilometer import service
