@@ -339,9 +339,18 @@ class PublisherTest(base.BaseTestCase):
     def test_activity_gnocchi_project_not_found(self, logger):
         self.ks_client.projects.find.side_effect = ka_exceptions.NotFound
         self._do_test_activity_filter(2)
-        logger.warning.assert_called_with('filtered project not found in '
-                                          'keystone, ignoring the '
-                                          'filter_project option')
+        logger.warning.assert_called_with(
+            'Filtered project [service] not found in keystone, ignoring the '
+            'filter_project option')
+
+    @mock.patch('ceilometer.publisher.gnocchi.GnocchiPublisher'
+                '._get_gnocchi_client')
+    def test_get_gnocchi_client(self, gnocchi_cli):
+        url = netutils.urlsplit("gnocchi://")
+        gnocchi_cli.side_effect = ka_exceptions.DiscoveryFailure
+        cfg = self.conf.conf
+        publisher = gnocchi.GnocchiPublisher
+        self.assertRaises(ka_exceptions.DiscoveryFailure, publisher, cfg, url)
 
     def test_activity_filter_match_swift_event(self):
         self.samples[0].name = 'storage.objects.outgoing.bytes'
@@ -375,24 +384,59 @@ class PublisherTest(base.BaseTestCase):
     @mock.patch('ceilometer.publisher.gnocchi.GnocchiPublisher'
                 '.batch_measures')
     def test_unhandled_meter_with_no_resource_id(self, fake_batch):
-        samples = [sample.Sample(
-            name='unknown.meter',
-            unit='GB',
-            type=sample.TYPE_GAUGE,
-            volume=2,
-            user_id='test_user',
-            project_id='test_project',
-            source='openstack',
-            timestamp='2014-05-08 20:23:48.028195',
-            resource_id=None,
-            resource_metadata={}
-        )]
+        samples = [
+            sample.Sample(
+                name='unknown.meter',
+                unit='GB',
+                type=sample.TYPE_GAUGE,
+                volume=2,
+                user_id='test_user',
+                project_id='test_project',
+                source='openstack',
+                timestamp='2014-05-08 20:23:48.028195',
+                resource_id=None,
+                resource_metadata={}),
+            sample.Sample(
+                name='unknown.meter',
+                unit='GB',
+                type=sample.TYPE_GAUGE,
+                volume=2,
+                user_id='test_user',
+                project_id='test_project',
+                source='openstack',
+                timestamp='2014-05-08 20:23:48.028195',
+                resource_id="Some-other-resource-id",
+                resource_metadata={})
+        ]
         url = netutils.urlsplit("gnocchi://")
         d = gnocchi.GnocchiPublisher(self.conf.conf, url)
         d._already_checked_archive_policies = True
         d.publish_samples(samples)
         self.assertEqual(0, len(fake_batch.call_args[0][1]))
 
+    @mock.patch('ceilometer.publisher.gnocchi.LOG')
+    @mock.patch('gnocchiclient.v1.client.Client')
+    def test__set_update_attributes_non_existent_resource(self,
+                                                          fakeclient_cls,
+                                                          logger):
+        url = netutils.urlsplit("gnocchi://")
+        self.publisher = gnocchi.GnocchiPublisher(self.conf.conf, url)
+
+        fakeclient = fakeclient_cls.return_value
+        fakeclient.resource.update.side_effect = [
+            gnocchi_exc.ResourceNotFound(404)]
+
+        non_existent_resource = {
+            'type': 'volume',
+            'id': self.resource_id,
+        }
+
+        self.publisher._set_update_attributes(non_existent_resource)
+
+        logger.debug.assert_called_with(
+            "Update event received on unexisting resource (%s), ignore it.",
+            self.resource_id)
+
 
 class MockResponse(mock.NonCallableMock):
     def __init__(self, code):
@@ -689,9 +733,12 @@ class PublisherWorkflowTest(base.BaseTestCase,
         for call in expected_calls:
             self.assertIn(call, fakeclient.mock_calls)
 
+    @mock.patch('ceilometer.cache_utils.get_client', mock.Mock())
     @mock.patch('ceilometer.publisher.gnocchi.LOG')
     @mock.patch('gnocchiclient.v1.client.Client')
     def test_workflow(self, fakeclient_cls, logger):
+        url = netutils.urlsplit("gnocchi://")
+        publisher = gnocchi.GnocchiPublisher(self.conf.conf, url)
 
         fakeclient = fakeclient_cls.return_value
 
@@ -722,12 +769,30 @@ class PublisherWorkflowTest(base.BaseTestCase,
                 {resource_id: {metric_name: self.metric_attributes}},
                 create_metrics=True)
         ]
+
+        resource_definition = publisher.metric_map.get(self.sample.name)
+
+        expected_measures_in_log = {resource_id: {self.sample.name: {
+            'measures': [{'timestamp': self.sample.timestamp,
+                          'value': self.sample.volume}],
+            'archive_policy_name': resource_definition.metrics[
+                metric_name]["archive_policy_name"],
+            'unit': self.sample.unit}}}
+
+        resource_type = resource_definition.cfg['resource_type']
+
         expected_debug = [
-            mock.call('filtered project found: %s',
+            mock.call('Filtered project [%s] found with ID [%s].', 'service',
                       'a2d42c23-d518-46b6-96ab-3fba2e146859'),
+            mock.call('Sample [%s] is not a Gnocchi activity; therefore, we '
+                      'do not filter it out and push it to Gnocchi.',
+                      self.sample),
             mock.call('Processing sample [%s] for resource ID [%s].',
                       self.sample, resource_id),
-        ]
+            mock.call('Executing batch resource metrics measures for resource '
+                      '[%s] and measures [%s].',
+                      mock.ANY,
+                      expected_measures_in_log)]
 
         measures_posted = False
         batch_side_effect = []
@@ -792,8 +857,6 @@ class PublisherWorkflowTest(base.BaseTestCase,
         batch = fakeclient.metric.batch_resources_metrics_measures
         batch.side_effect = batch_side_effect
 
-        url = netutils.urlsplit("gnocchi://")
-        publisher = gnocchi.GnocchiPublisher(self.conf.conf, url)
         publisher.publish_samples([self.sample])
 
         # Check that the last log message is the expected one
@@ -801,7 +864,17 @@ class PublisherWorkflowTest(base.BaseTestCase,
                 or self.create_resource_fail
                 or self.retry_post_measures_fail
                 or (self.update_resource_fail and self.patchable_attributes)):
-            logger.error.assert_called_with('boom!', exc_info=True)
+
+            if self.update_resource_fail and self.patchable_attributes:
+                logger.error.assert_called_with(
+                    'Unexpected exception updating resource type [%s] with '
+                    'ID [%s] for resource data [%s]: [%s].', resource_type,
+                    resource_id, mock.ANY, 'boom!', exc_info=True)
+            else:
+                logger.error.assert_called_with(
+                    'Unexpected exception while pushing measures [%s] for '
+                    'gnocchi data [%s]: [%s].', expected_measures_in_log,
+                    mock.ANY, 'boom!', exc_info=True)
         else:
             self.assertEqual(0, logger.error.call_count)
         self.assertEqual(expected_calls, fakeclient.mock_calls)
